﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZooContracts.BusinessLogicsContracts;
using ZooContracts.SearchModel;
using ZooContracts.StoragesContracts;
using ZooContracts.ViewModels;
using ZooDatabaseImplements.Implements;


namespace ZooBusinessLogic.BusinessLogics
{
    public class CostLogic : ICostLogic
    {
        private readonly ILogger _logger;
        private readonly ICostStorage _CostStorage;
        public CostLogic(ILogger<RouteCostLogic> logger, ICostStorage CostStorage)
        {
            _logger = logger;
            _CostStorage = CostStorage;
        }

        public bool Create(CostBindingModel model)
        {
            CheckModel(model);
            if (_CostStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(CostBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_CostStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public CostViewModel? ReadElement(CostSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id: {Id}", model.Id);
            var element = _CostStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
            return element;
        }

        public List<CostViewModel>? ReadList(CostSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);
            var list = model == null ? _CostStorage.GetFullList() : _CostStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool Update(CostBindingModel model)
        {
            CheckModel(model);
            if (_CostStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }
        private void CheckModel(CostBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            _logger.LogInformation("Cost. Id: {Id}", model.Id);
        }
    }
}
