﻿using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using ZooContracts.BusinessLogicsContracts;
using ZooContracts.SearchModel;
using ZooContracts.StoragesContracts;
using ZooContracts.ViewModels;
using ZooDatabaseImplements.Implements;

namespace ZooBusinessLogic.BusinessLogics
{
    public class RouteReserveLogic : IRouteReserveLogic
    {
        private readonly ILogger _logger;
        private readonly IRouteReserveStorage _RouteReserveStorage;
        public RouteReserveLogic(ILogger<RouteReserveLogic> logger, IRouteReserveStorage RouteReserveStorage)
        {
            _logger = logger;
            _RouteReserveStorage = RouteReserveStorage;
        }

        public bool Create(RouteReserveBindingModel model)
        {
            CheckModel(model);
            if (_RouteReserveStorage.Insert(model) == null)
            {
                _logger.LogWarning("Insert operation failed");
                return false;
            }
            return true;
        }

        public bool Delete(RouteReserveBindingModel model)
        {
            CheckModel(model, false);
            _logger.LogInformation("Delete. Id: {Id}", model.Id);
            if (_RouteReserveStorage.Delete(model) == null)
            {
                _logger.LogWarning("Delete operation failed");
                return false;
            }
            return true;
        }

        public RouteReserveViewModel? ReadElement(RouteReserveSearchModel model)
        {
            if (model == null)
            {
                throw new ArgumentNullException(nameof(model));
            }
            _logger.LogInformation("ReadElement. Id: {Id}", model.Id);
            var element = _RouteReserveStorage.GetElement(model);
            if (element == null)
            {
                _logger.LogWarning("ReadElement element not found");
                return null;
            }
            _logger.LogInformation("ReadElement found. Id: {Id}", element.Id);
            return element;
        }

        public List<RouteReserveViewModel>? ReadList(RouteReserveSearchModel? model)
        {
            _logger.LogInformation("ReadList. Id: {Id}", model?.Id);
            var list = model == null ? _RouteReserveStorage.GetFullList() : _RouteReserveStorage.GetFilteredList(model);
            if (list == null)
            {
                _logger.LogWarning("ReadList return null list");
                return null;
            }
            _logger.LogInformation("ReadList. Count: {Count}", list.Count);
            return list;
        }

        public bool Update(RouteReserveBindingModel model)
        {
            CheckModel(model);
            if (_RouteReserveStorage.Update(model) == null)
            {
                _logger.LogWarning("Update operation failed");
                return false;
            }
            return true;
        }

        private void CheckModel(RouteReserveBindingModel model, bool withParams = true)
        {
            if (model == null)
            {
                throw new ArgumentException(nameof(model));
            }
            if (!withParams)
            {
                return;
            }
            _logger.LogInformation("RouteReserve. Id: {Id}", model.Id);
        }
    }
}
